/**
 * Lukasz 'Severiaan' Grela
 * greladesign © 2011
 * Copies filter stack of selected object and replicates it in AS2/AS3 actionscript code,
 * uses ColorMatrix class from http://gskinner.com/blog/archives/2005/09/flash_8_source_.html for adjustColorFilter with permission from Grant Skinner.
 */

//-------------------------------------
// Data
//-------------------------------------
/**
 * List of IDE properties (Filter object) mapped against filter names that are using it
 */
var propertyMap = [];
	propertyMap[ 0] = [["angle"], 			["bevelFilter", "dropShadowFilter", "gradientBevelFilter", "gradientGlowFilter"]];
	propertyMap[ 1] = [["blurX"], 			["bevelFilter", "blurFilter", "dropShadowFilter", "glowFilter", "gradientBevelFilter", "gradientGlowFilter"]];
	propertyMap[ 2] = [["blurY"], 			["bevelFilter", "blurFilter", "dropShadowFilter", "glowFilter", "gradientBevelFilter", "gradientGlowFilter"]];
	propertyMap[ 3] = [["brightness"], 		["adjustColorFilter"]];
	propertyMap[ 4] = [["color"], 			["dropShadowFilter", "glowFilter"]];
	propertyMap[ 5] = [["contrast"], 		["adjustColorFilter"]];
	propertyMap[ 6] = [["distance"], 		["bevelFilter", "dropShadowFilter", "gradientBevelFilter", "gradientGlowFilter"]];
	propertyMap[ 7] = [["hideObject"], 		["dropShadowFilter"]];
	propertyMap[ 8] = [["highlightColor"], 	["bevelFilter"]];
	propertyMap[ 9] = [["hue"], 			["adjustColorFilter"]];
	propertyMap[10] = [["inner"], 			["dropShadowFilter", "glowFilter"]];
	propertyMap[11] = [["knockout"], 		["bevelFilter", "dropShadowFilter", "glowFilter", "gradientBevelFilter", "gradientGlowFilter"]];
	propertyMap[12] = [["quality"], 		["bevelFilter", "blurFilter", "dropShadowFilter", "glowFilter", "gradientGlowFilter", "gradientBevelFilter"]];
	propertyMap[13] = [["saturation"], 		["adjustColorFilter"]];
	propertyMap[14] = [["shadowColor"], 	["bevelFilter"]];
	propertyMap[15] = [["strength"], 		["bevelFilter", "dropShadowFilter", "glowFilter", "gradientGlowFilter", "gradientBevelFilter"]];
	propertyMap[16] = [["type"], 			["bevelFilter", "gradientGlowFilter", "gradientBevelFilter"]];
	
	//additionals
	propertyMap[17] = [["colorArray"], 		["gradientGlowFilter", "gradientBevelFilter"]];
	propertyMap[18] = [["gradientIndex"],	["gradientGlowFilter", "gradientBevelFilter"]];
	propertyMap[19] = [["posArray"], 		["gradientGlowFilter", "gradientBevelFilter"]];
/**
 * Hash map that will help keep unwanted imports away, not repeated for each filter in stack.
 */
var importStatementMap = [];

var autohringVersion = fl.version.split(" ")[1];

var m_bUseAS2Version = parseInt(autohringVersion.split(",")[0]) <= 8;

var m_oAdjustColorFilter = {brightness:0,contrast:0,saturation:0,hue:0};

//-------------------------------------
// Application
//-------------------------------------
//TODO: wrap in some kind of interface

var listOfFilters = document.getFilters();
var m_nFilterIndex = 0;
var m_aFilterIds = [];
fl.trace("//Filters found: "+listOfFilters.length);
for(var i=0; i<listOfFilters.length; i++)
{
	fl.trace("//----------------------------//");
	fl.trace("//Filter#"+i);
	fl.trace("//----------------------------//");
	m_nFilterIndex = i;
	if(listOfFilters[i].enabled || m_bUseAS2Version) m_aFilterIds.push(getFilterInstanceName());
	getFilter(listOfFilters[i]);
}
if(listOfFilters.length > 0)
{
	fl.trace("//<DisplayObject>.filters = ["+m_aFilterIds+"];")
	fl.trace("//----------------------------//");
}
//-------------------------------------
// Functions
//-------------------------------------
/**
 * Returns filter instance name according to the order in filters list
 */
function getFilterInstanceName()
{
	return "f" + m_nFilterIndex;
}


/**
 * Outputs AS3/AS2 code version of filter applied to an object
 * @param	filter
 */
function getFilter(filter)
{
	if (getImport(filter.name))
	{
		var props = getFilterProps(filter.name);
		flash.trace("//"+props);
		/*
		for(var i in filter)
		{
			//fl.trace(i + "=" + filter[i]);
			getValue(filter.name, i, filter[i]);
		}
		*/
		
		for (var i = 0; i < props.length; i++)
		{
			var property = props[i];
			getValue(filter.name, property, filter[property]);
		}
		if(filter.name == "adjustColorFilter")
		{

			var arr = fl.runScript(fl.configURI + "Commands/gskinner_color_matrix.jsfl", "adjustColor"
												
												, m_oAdjustColorFilter.brightness
												, m_oAdjustColorFilter.contrast
												, m_oAdjustColorFilter.saturation
												, m_oAdjustColorFilter.hue
												
												);
			//fl.trace("arr:"+arr);
			var matName = "mat" + m_nFilterIndex;
			fl.trace("var "+matName+":Array = ["+arr+"]");
			fl.trace(getFilterInstanceName()+".matrix = "+matName+";");
			/*			*/
		}
	}
	else
	{
		//debug output
		//*
		for(var i in filter)
		{
			fl.trace("//"+i + "=" + filter[i]);
		}
		//*/
	}
}
/**
 * Returns list of expected properties to be searched on filters object
 * NOTE: alpha property is part of the color (in RGBA format) from IDE
 * @param	filterName
 */
function getFilterProps(filterName)
{
	return getSupportedProperties(filterName);
}
/**
 * Outputs import statements for given filter
 * @param	filterName
 */
function getImport(filterName)
{
	var _sDefaultImport = "import flash.filters.BitmapFilterQuality;";
	var _sImport = "";
	var _sExtraImport = "";
	var _sInit = ""
	switch(filterName)
	{
		case "glowFilter":
			_sImport = "import flash.filters.GlowFilter;";
			_sInit = "var "+getFilterInstanceName()+":GlowFilter = new GlowFilter();";
		break;
		case "dropShadowFilter":
			_sImport = "import flash.filters.DropShadowFilter;";
			_sInit = "var "+getFilterInstanceName()+":DropShadowFilter = new DropShadowFilter();";
		break;
		case "blurFilter":
			_sImport = "import flash.filters.BlurFilter;";
			_sInit = "var "+getFilterInstanceName()+":BlurFilter = new BlurFilter();"
		break;
		case "bevelFilter":
			_sImport = "import flash.filters.BevelFilter;"
			if(!m_bUseAS2Version) _sExtraImport = "import flash.filters.BitmapFilterType;";
			_sInit = "var "+getFilterInstanceName()+":BevelFilter = new BevelFilter();"
		
		break;
		case "adjustColorFilter":
			//
			_sImport = "import flash.filters.ColorMatrixFilter;"
			
			_sInit = "var "+getFilterInstanceName()+":ColorMatrixFilter = new ColorMatrixFilter();"
		break;
		case "gradientGlowFilter":
			_sImport = "import flash.filters.GradientGlowFilter;"
			if(!m_bUseAS2Version) _sExtraImport = "import flash.filters.BitmapFilterType;";
			_sInit = "var "+getFilterInstanceName()+":GradientGlowFilter = new GradientGlowFilter();"
		break;
		case "gradientBevelFilter":
			_sImport = "import flash.filters.GradientBevelFilter;"
			if(!m_bUseAS2Version) _sExtraImport = "import flash.filters.BitmapFilterType;";
			_sInit = "var "+getFilterInstanceName()+":GradientBevelFilter = new GradientBevelFilter();";
		break;
		default:
			flash.trace("Filter '" + filterName + "' is not supported yet");
			return false;
	}
	
	if(!m_bUseAS2Version)
	{
		if (importStatementMap[_sDefaultImport] != true)
		{
			fl.trace(_sDefaultImport);
			importStatementMap[_sDefaultImport] = true;
		}
	}
	
	if (importStatementMap[_sExtraImport] != true && _sExtraImport != "")
	{
		fl.trace(_sExtraImport);
		importStatementMap[_sExtraImport] = true;
	}
	
	if (importStatementMap[_sImport] != true)
	{
		fl.trace(_sImport);
		importStatementMap[_sImport] = true;
	}
	fl.trace("");
	fl.trace(_sInit);
	fl.trace("");
	
	return true;
}

/**
 * Translates string representation of filter quality into values of BitmapFilterQuality class;
 * @param	value
 */
function getFilterQuality(value)
{
	switch (value)
	{
		case "high":
			if(m_bUseAS2Version) fl.trace(getFilterInstanceName() + "." + "quality = 3;");
			else fl.trace(getFilterInstanceName() + "." + "quality = BitmapFilterQuality.HIGH;");
		break;
		case "medium":
			if(m_bUseAS2Version) fl.trace(getFilterInstanceName() + "." + "quality = 2;");
			else fl.trace(getFilterInstanceName() + "." + "quality = BitmapFilterQuality.MEDIUM;");
		break;
		case "low":
		default:
			if(m_bUseAS2Version) fl.trace(getFilterInstanceName() + "." + "quality = 1;");
			else fl.trace(getFilterInstanceName() + "." + "quality = BitmapFilterQuality.LOW;");
	}
}
/**
 * Translates the string representation of filter type into values of BitmapFilterType class;
 * @param	value
 */
function getBitmapType(value)
{
	switch (value)
	{
		case "full":
			if(m_bUseAS2Version) fl.trace(getFilterInstanceName() + "." + "type = \"full\";");
			else fl.trace(getFilterInstanceName() + "." + "type = BitmapFilterType.FULL;");
		break;
		case "outer":
			if(m_bUseAS2Version) fl.trace(getFilterInstanceName() + "." + "type = \"outer\";");
			else fl.trace(getFilterInstanceName() + "." + "type = BitmapFilterType.OUTER;");
		break;
		case "inner":
		default:
			if(m_bUseAS2Version) fl.trace(getFilterInstanceName() + "." + "type = \"inner\";");
			else fl.trace(getFilterInstanceName() + "." + "type = BitmapFilterType.INNER;");
	}
}
/**
 * Returns Color from html formatted rgba format into  numerical rgb i.e. #FF0000FF - > 0xFF0000 (red)
 * @param	color
 */
function getColor(color)
{
	var _sHex = "0x";
	
	if(color.indexOf("'") != -1)
	{
		//remove quotation mark
		color = color.split("'").join("");
	}
	
	if (color.length == 7) color = color.substr(1);
	if (color.length == 9) color = color.substr(1,6);
	
	return _sHex + color;
}
/**
 * Returns number from 0..1 representing alpha part of color value passed
 * @param	color
 */
function getAlpha(color)
{
	//we have value from 0..255 which needs to be converted to range 0..1
	
	if(color.indexOf("'") != -1)
	{
		//remove quotation mark
		color = color.split("'").join("");
	}
	
	if (color.length == 7) return 1;
	else
	{
		var _sAlphaHex = color.substr(7, 2);
		//now convert into a value from 0..1 with little bit of rounding
		var _nPrcnt = Math.round((parseInt(_sAlphaHex, 16) / 0xFF) * 100) / 100;
		return _nPrcnt;
	}
}
/**
 * Outputs filter property adjusting value as appriopriate
 *
 * @param	filterName
 * @param	property
 * @param	value
 */
function getValue(filterName, property, value)
{
	//flash.trace("getValue(filterName="+filterName+", property="+property+", value="+value+")")
	switch (property)
	{
		case "angle":
		case "blurX":
		case "blurY":
		case "distance":
			if(property == "angle")
			{
				value = Math.round(value * 100) / 100;
			}
			flash.trace(getFilterInstanceName() + "." + property + " = " + value + ";");
		break;
		case "brightness":
			m_oAdjustColorFilter[property] = value;
		break;
		case "color":
			//tricky as it can contain alpha value
			var rgb = getColor(value);
			var alpha = getAlpha(value);
			getValue(filterName, "alpha", alpha);
			flash.trace(getFilterInstanceName() + "." + property + " = " + rgb + ";");
		break;
		case "alpha":
			flash.trace(getFilterInstanceName() + "." + property + " = " + value + ";");
		break;
		case "contrast":
			m_oAdjustColorFilter[property] = value;
		break;
		case "enabled":
		break;
		case "highlightAlpha":
			flash.trace(getFilterInstanceName() + "." + property + " = " + value + ";");
		break;
		case "highlightColor":
			var rgb = getColor(value);
			var alpha = getAlpha(value);
				getValue(filterName, "highlightAlpha", alpha);
			flash.trace(getFilterInstanceName() + "." + property + " = " + rgb + ";");
		break;
		
		case "hue":
			m_oAdjustColorFilter[property] = value;
		break;
		
		case "hideObject":
		case "inner":
		case "knockout":
			flash.trace(getFilterInstanceName() + "." + property + " = " + value + ";");
		break;
		
		case "name"://ignore
		break;
		
		case "quality":
			getFilterQuality(value);
		break;
		
		case "saturation":
			m_oAdjustColorFilter[property] = value;
		break;
		
		case "shadowAlpha":
			flash.trace(getFilterInstanceName() + "." + property + " = " + value + ";");
		break;
		case "shadowColor":
		
			var rgb = getColor(value);
			var alpha = getAlpha(value);
			getValue(filterName, "shadowAlpha", alpha);
			flash.trace(getFilterInstanceName() + "." + property + " = " + rgb + ";");
			
		break;
		case "strength":
			flash.trace(getFilterInstanceName() + "." + property + " = " + (parseInt(value) / 100) + ";");
		break;
		case "type":
			getBitmapType(value);
		break;
		
		case "posArray":
				//flash.trace("//unknown property: " + property + " = " + value + ";");
				//ratios
				flash.trace(getFilterInstanceName() + "." + "ratios" + " = [" + value + "];");
				
		break;
		case "alphaArray":
				//alphas
				var _aAlpha = value.concat();
				for (var i = 0; i < _aAlpha.length; i++)
				{
					var a = getAlpha(_aAlpha[i]);
					_aAlpha[i] = a;
				}
			
			flash.trace(getFilterInstanceName() + "." + "alphas" + " = [" + _aAlpha + "];");
			
		break;
		case "colorArray":
				//colors
				var _aRGB = value.concat();
				for (var i = 0; i < _aRGB.length; i++)
				{
					var color = getColor(_aRGB[i]);
					_aRGB[i] = color;
				}
			
			flash.trace(getFilterInstanceName() + "." + "colors" + " = [" + _aRGB + "];");
			
			getValue(filterName, "alphaArray", value);
		break;
		case "gradientIndex":
			//ignore as it seems to do nothing
		break;
		default:
			
				flash.trace("//unknown property: " + property + " = " + value + ";");
	}
}
/**
 * Returns list of properties that given filter supports
 *
 * @param	property
 * @param	filterName
 */
function getSupportedProperties(filterName)
{
	var _aProps = [];
	for (var i = 0; i < propertyMap.length; i++)
	{
		var propName = propertyMap[i][0][0];
		
			for (var j = 0; j < propertyMap[i][1].length; j++)
			{
				var filter = propertyMap[i][1][j];
				
				if (filter == filterName)
				{
					_aProps.push(propName);
				}
			}
		
	}
	return _aProps;
}
